﻿using Sirenix.Utilities;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;
using UnityEditor;
using UnityEngine;

/*
 *Copyright(C) 2020 by  Chief All rights reserved.
 *Unity版本：2019.4.8f1 
 *作者:Chief  
 *创建日期: 2020-08-17 
 *模块说明：
 *版本: 1.0.0
*/

namespace Lycianthes
{

    public class CreateScriptByTemplate
    {
        public static string Version = "0.0.1";
        public static string NameSpace = "RedPanda";
        public static string MoudleInfo = "模块说明";

        public static void CreateSript(string resourceFile, string fileName = "/Scripts/", string className = "New_CS_Script")
        {

            //选取路径
            string path = EditorUtility.SaveFilePanel("Create Script", Application.dataPath + fileName, className, "cs");

            if (path.IsNullOrWhitespace())
            {
#if DEBUG_LYCIANTHES
                Debug.Log("脚本创建已取消");
#endif
                return;
            }

            StreamReader streamReader = new StreamReader(resourceFile);
            string text = streamReader.ReadToEnd();
            streamReader.Close();

            int index = path.LastIndexOf('/') + 1;
            className = path.Substring(index);
            className = className.Substring(0, className.Length - 3);
            if (string.IsNullOrEmpty(path))
            {
#if DEBUG_LYCIANTHES
                Debug.LogError("save path error!");
#endif
                return;
            }
            Debug.Log("创建脚本：" + path);
            //设置文本
            text = SetRiseDesc(text);
            text = Regex.Replace(text, "#SCRIPTNAME#", className);
            text = Regex.Replace(text, "#NAMESPACE#", NameSpace);

            bool encoderShouldEmitUTF8Identifier = true;
            bool throwOnInvalidBytes = false;
            UTF8Encoding encoding = new UTF8Encoding(encoderShouldEmitUTF8Identifier, throwOnInvalidBytes);
            bool append = false;
            StreamWriter streamWriter = new StreamWriter(path, append, encoding);
            streamWriter.Write(text);
            streamWriter.Close();

            Debug.Log("<color=green>脚本已经自动创建，需要让Unity重新获得一次焦点来激活</color>");
        }

        public static void CreateScriptNoChosse(string resourceFile, string fileName, string className)
        {
            StreamReader streamReader = new StreamReader(resourceFile);
            string text = streamReader.ReadToEnd();
            streamReader.Close();

            string path = $"{Application.dataPath}{fileName}/{className}.cs";

            int index = path.LastIndexOf('/') + 1;
            className = path.Substring(index);
            className = className.Substring(0, className.Length - 3);
            if (string.IsNullOrEmpty(path))
            {
#if DEBUG_LYCIANTHES
                Debug.LogError("save path error!");
#endif
                return;
            }
            Debug.Log("创建脚本：" + path);
            //设置文本
            text = SetRiseDesc(text);
            text = Regex.Replace(text, "#SCRIPTNAME#", className);
            text = Regex.Replace(text, "#NAMESPACE#", NameSpace);

            bool encoderShouldEmitUTF8Identifier = true;
            bool throwOnInvalidBytes = false;
            UTF8Encoding encoding = new UTF8Encoding(encoderShouldEmitUTF8Identifier, throwOnInvalidBytes);
            bool append = false;
            StreamWriter streamWriter = new StreamWriter(path, append, encoding);
            streamWriter.Write(text);
            streamWriter.Close();

            Debug.Log("<color=green>脚本已经自动创建，需要让Unity重新获得一次焦点来激活</color>");
        }

        /// <summary>
        /// 设置抬头信息
        /// </summary>
        static string SetRiseDesc(string text)
        {
            StringBuilder summary = new StringBuilder();
            summary.Append("/*\n");
            summary.Append($" *Copyright(C) 2023 by {LycianthesConfig.GetInstance().CompanyName} All rights reserved.\n");
            summary.Append($" *Unity版本：{Application.unityVersion} \n");
            summary.Append($" *作者:{LycianthesConfig.GetInstance().AuthorName}  \n");
            summary.Append($" *创建日期: {System.DateTime.Now.ToString("yyyy-MM-dd")} \n");
            if (LycianthesConfig.GetInstance().UseModuleInfo)
                summary.Append($" *模块说明：{MoudleInfo}\n");
            summary.Append($" *版本: {Version}\n");
            summary.Append("*/");
            text = Regex.Replace(text, "#SUMMARY#", summary.ToString());
            return text;
        }

    }
}