﻿/*
 *Copyright(C) 2023 by Chief All rights reserved.
 *Unity版本：2020.3.41f1c1 
 *作者:Chief  
 *创建日期: 2022-12-06 
 *模块说明：
 *版本: 1.0
*/

using Sirenix.OdinInspector;
using Sirenix.OdinInspector.Editor;
using Sirenix.Utilities;
using Sirenix.Utilities.Editor;
using UnityEditor;
using UnityEngine;

namespace Lycianthes
{
    /// <summary>
    /// 配置窗口
    /// </summary>
    public class LycianthesConfigWindow : OdinEditorWindow
    {
        /// <summary>
        /// 上次选择的默认脚本模板文件夹
        /// </summary>
        public const string KEY_LYCIANTHES_LAST_TEMPLATE_FOLDER = "KEY_LYCIANTHES_LAST_TEMPLATE_FOLDER";

        [MenuItem("Tools/Lycianthes/本地设置")]
        public static void ShowWindow()
        {
            var window = GetWindow<LycianthesConfigWindow>();
            window.position = GUIHelper.GetEditorWindowRect().AlignCenter(500, 375);
            window.titleContent = new GUIContent("本地配置");

            window.LocalConfig = LycianthesConfig.GetInstance();

            if (PlayerPrefs.HasKey(KEY_LYCIANTHES_LAST_TEMPLATE_FOLDER))
                window.ScriptTemplateFolder = PlayerPrefs.GetString(KEY_LYCIANTHES_LAST_TEMPLATE_FOLDER);
        }

        [Title("")]
        [PropertyOrder(10)]
        [ShowInInspector, HideLabel]
        public LycianthesConfig LocalConfig;

        [Title("")]
        [Button("确认并保存", ButtonSizes.Medium)]
        [PropertyOrder(20)]
        public void Save()
        {
            if (LocalConfig == null) return;
            PlayerPrefs.SetString(LycianthesConfig.KEY_LYCIANTHES_CONFIG, LocalConfig.GetJsonString());
        }

        [Title("脚本模板创建")]
        [FolderPath]
        [PropertyOrder(30)]
        [PropertySpace(25)]
        [LabelText("脚本模板文件夹")]
        public string ScriptTemplateFolder;

        /// <summary>
        /// 是否有合法的脚本模板文件夹
        /// </summary>
        private bool HasScriptTemplateFolder { get { return !string.IsNullOrEmpty(ScriptTemplateFolder); } }

        [LabelText("脚本模板文件名")]
        [PropertyOrder(30)]
        [ShowIf("HasScriptTemplateFolder")]
        public string ScriptTemplateAssetName;

        /// <summary>
        /// 创建脚本模板
        /// </summary>
        [PropertyOrder(30)]
        [ShowIf("HasScriptTemplateFolder")]
        [Button("创建脚本模板", ButtonSizes.Medium)]
        public void CreateScriptTemplate()
        {
            if (!HasScriptTemplateFolder) return;

            if (string.IsNullOrEmpty(ScriptTemplateAssetName))
            {
                EditorUtility.DisplayDialog("Lycianthes配置", "脚本模板文件名不能为空！", "确定");
                return;
            }

            string path = $"{ScriptTemplateFolder}/{ScriptTemplateAssetName}.asset";
            //先加载一下是否有当前文件；
            var template = AssetDatabase.LoadAssetAtPath<LycianthesTemplateGroup>(path);
            if (template != null)
            {
                EditorUtility.DisplayDialog("Lycianthes配置", $"已经有此模板：{ScriptTemplateAssetName}\n不能重复创建！", "确定");
                return;
            }

            template = CreateInstance<LycianthesTemplateGroup>();
            AssetDatabase.CreateAsset(template, path);
            Debug.Log($"已生成模板：{path} : {template.name}");
            //缓存本次保存的文件夹
            PlayerPrefs.SetString(KEY_LYCIANTHES_LAST_TEMPLATE_FOLDER, ScriptTemplateFolder);
        }

    }
}