﻿/*
 *Copyright(C) 2020 by  Chief All rights reserved.
 *Unity版本：2019.4.8f1 
 *作者:Chief  
 *创建日期: 2020-08-17 
 *模块说明：
 *版本: 1.0.0
*/

using Sirenix.OdinInspector;
using System;
using System.Collections.Generic;
using UnityEditor;
using UnityEngine;

namespace Lycianthes
{

    //[CreateAssetMenu(fileName = "RedPandScriptCreator", menuName = "脚本创建器", order = 1)]
    public class LycianthesScriptCreator : SerializedScriptableObject
    {
        #region 自动获取配置方法

        private static LycianthesScriptCreator _Instance;
        public static LycianthesScriptCreator Instance
        {
            get
            {
                if (_Instance == null)
                    AutoLoadAsset();
                return _Instance;
            }
        }

        /// <summary>
        /// 自动获取路径的方法；
        /// </summary>
        [InitializeOnLoadMethod]
        private static void AutoLoadAsset()
        {
            string assetName = "LycianthesScriptCreator";
            string[] guids = AssetDatabase.FindAssets(assetName);

            foreach (var guid in guids)
            {
                string path = AssetDatabase.GUIDToAssetPath(guid);
                if (path.EndsWith(".asset"))
                {
                    LycianthesScriptWindow.AssetPath = path;
                    _Instance = AssetDatabase.LoadAssetAtPath<LycianthesScriptCreator>(path);
#if DEBUG_LYCIANTHES
                    Debug.Log($"[Lycianthes] : 配置路径{path} |{_Instance}");
#endif
                    return;
                }
            }
#if DEBUG_LYCIANTHES
            Debug.LogError($"[Lycianthes] : 未找到目标资源：{assetName}");
#endif
        }

        #endregion

        [LabelText("版本")]
        public string Version = "1.0.0";

        [LabelText("用户名")]
        [ShowIf("ShowMoudleInfo")]
        public string MoudleInfo = "模块说明";

        private bool ShowMoudleInfo => LycianthesConfig.GetInstance().UseModuleInfo;

        [LabelText("命名空间")]
        public string NameSpace = "RedPanda";

        [Title("预设脚本配置"), HideLabel]
        public Dictionary<string, LycianthesScriptInfo> dictTemplate = new Dictionary<string, LycianthesScriptInfo>();

        /// <summary>
        /// 其他脚本的配置
        /// </summary>
        [NonSerialized]
        private List<LycianthesTemplateGroup> listOtherTemplate = new List<LycianthesTemplateGroup>();

        public void ResetOtherTemplate() { listOtherTemplate?.Clear(); }

        /// <summary>
        /// 新建脚本
        /// </summary>
        [Title("脚本创建"), Button("创建脚本", ButtonSizes.Medium)]
        public void CreateNewScript(string type)
        {
            LycianthesScriptInfo temp = FindLycianthesScriptInfo(type);
            if (temp == null) return;

            //获取文件路径；
            string path = AssetDatabase.GetAssetPath(temp.Template);
            //创建脚本
            CreateScriptByTemplate.Version = Version;
            CreateScriptByTemplate.MoudleInfo = MoudleInfo;
            CreateScriptByTemplate.NameSpace = NameSpace;
            //删除前面的/Asset
            string curPath = temp.DefaultFolder.Substring(6);
            CreateScriptByTemplate.CreateSript(path, curPath, temp.DefaultName);
        }

        public LycianthesScriptInfo FindLycianthesScriptInfo(string type)
        {
            LycianthesScriptInfo ret = null;
            if (dictTemplate.TryGetValue(type, out ret)) return ret;
            foreach (var group in listOtherTemplate)
            {
                if (group.dictTemplate.TryGetValue(type, out ret))
                    return ret;
            }
#if DEBUG_LYCIANTHES
            Debug.LogError($"[Lycianthes] : 未找到对应脚本配置：{type}");
#endif
            return null;
        }

        public void CreateNewScript(string type, string curPath, string name)
        {
            LycianthesScriptInfo temp = FindLycianthesScriptInfo(type);
            if (temp == null) return;

            //获取文件路径；
            string path = AssetDatabase.GetAssetPath(temp.Template);
            //创建脚本
            CreateScriptByTemplate.Version = Version;
            CreateScriptByTemplate.MoudleInfo = MoudleInfo;
            CreateScriptByTemplate.NameSpace = NameSpace;
            //删除前面的/Asset
            CreateScriptByTemplate.CreateScriptNoChosse(path, curPath, name);
        }

        /// <summary>
        /// 刷新；
        /// </summary>
        [Button("刷新")]
        public void Refresh() { AssetDatabase.Refresh(); }

        public void AddTemlateGroup(LycianthesTemplateGroup group)
        {
            if (group == null) return;

            if (listOtherTemplate.Contains(group)) return;
            listOtherTemplate.Add(group);
        }

        private List<string> tempKeyList = new List<string>();
        public List<string> GetScriptTemplateKey()
        {
            tempKeyList.Clear();
            foreach (var item in dictTemplate)
                tempKeyList.Add(item.Key);
            foreach (var item in listOtherTemplate)
                foreach (var info in item.dictTemplate)
                    tempKeyList.Add(info.Key);
            return tempKeyList;
        }

    }
}