﻿/*
 *Copyright(C) 2020 by  Chief All rights reserved.
 *Unity版本：2019.4.8f1 
 *作者:Chief  
 *创建日期: 2020-08-27 
 *模块说明：
 *版本: 1.0.0
*/

using Sirenix.OdinInspector;
using Sirenix.OdinInspector.Editor;
using Sirenix.Utilities;
using Sirenix.Utilities.Editor;
using System.Collections.Generic;
using System.Reflection;
using UnityEditor;
using UnityEngine;

namespace Lycianthes
{
    /// <summary>
    /// 创建脚本的窗口；
    /// </summary>
    public class LycianthesScriptWindow : OdinEditorWindow
    {
        public const string KEY_USER_NAMESPACE = "KEY_SCRIPTCREATOR_USER_NAMESPACE";
        public const string KEY_USER_MOUDLE_INFO = "KEY_SCRIPTCREATOR_USER_MOUDLE_INFO";
        public const float LABEL_WIDTH = 60;

        [MenuItem("Assets/Create/创建.CS脚本", false, 81)]
        public static void ShowWindow()
        {
            var window = GetWindow<LycianthesScriptWindow>();
            window.position = GUIHelper.GetEditorWindowRect().AlignCenter(500, 375);
            window.titleContent = new GUIContent("脚本模板创建");

            //缓存的配置文件位置
            window.creator = AssetDatabase.LoadAssetAtPath<LycianthesScriptCreator>(window.assetPath);
            if (PlayerPrefs.HasKey(KEY_USER_NAMESPACE))
                window.NameSpace = PlayerPrefs.GetString(KEY_USER_NAMESPACE);
            //当前的文件夹；
            window.TargetPath = GetCurrentSelectFolderPath();
            window.Version = PlayerSettings.bundleVersion;
            window.creator.ResetOtherTemplate();
            AutoLoadTemplateGroup();
        }

        private static void AutoLoadTemplateGroup()
        {
            var guids = AssetDatabase.FindAssets("t:LycianthesTemplateGroup");
            foreach (var guid in guids)
            {
                var path = AssetDatabase.GUIDToAssetPath(guid);
                if (path.EndsWith(".asset"))
                {
                    var group = AssetDatabase.LoadAssetAtPath<LycianthesTemplateGroup>(path);
                    group.OnEnable();
                }
            }
        }

        /// <summary>
        /// 获取当前选中的文件夹路径；
        /// </summary>
        /// <returns></returns>
        public static string GetCurrentSelectFolderPath()
        {
            var assembly = Assembly.GetAssembly(typeof(UnityEditor.ProjectWindowUtil));
            var type = assembly.GetType("UnityEditor.ProjectBrowser");

            if (type == null)
                return null;

            var browserField = type.GetField("s_LastInteractedProjectBrowser", BindingFlags.Public | BindingFlags.Static);
            var browser = browserField.GetValue(null);
            if (browser == null)
                return null;

            var folderPath = type.GetField("m_SelectedPath", BindingFlags.NonPublic | BindingFlags.Instance);
            return folderPath.GetValue(browser).ToString();
        }

        public static string AssetPath;

        [DisplayAsString, LabelText("配置文件位置")]
        [LabelWidth(LABEL_WIDTH)]
        [Title("配置信息")]
        [ShowInInspector]
        [PropertyOrder(0)]
        private string assetPath
        {
            get { return AssetPath; }
            set { }
        }

        /// <summary>
        /// 资源位置
        /// </summary>
        [LabelText("配置目录"), ReadOnly]
        [LabelWidth(LABEL_WIDTH)]
        [PropertyOrder(1)]
        [HideReferenceObjectPicker]
        public LycianthesScriptCreator creator;

        /// <summary>
        /// 命名空间；
        /// </summary>
        [Title("脚本说明配置", "本地配置菜单： Tools/Lycianthes/本地设置", TitleAlignment = TitleAlignments.Split)]
        [LabelText("命名空间"), OnValueChanged("OnChangeUserSetting")]
        [LabelWidth(LABEL_WIDTH)]
        [PropertyOrder(10)]
        public string NameSpace = "RedPanda";

        /// <summary>
        /// 版本号，版本号应该是同一控制
        /// </summary>
        [LabelText("版本号")]
        [LabelWidth(LABEL_WIDTH)]
        [PropertyOrder(10)]
        public string Version = "1.0.0";

        /// <summary>
        /// 公司名，读取的PlayerSetting;
        /// </summary>
        [LabelText("模板信息")]
        [LabelWidth(LABEL_WIDTH)]
        [ShowIf("ShowMoudleInfo")]
        [PropertyOrder(10)]
        [OnValueChanged("OnChangeUserSetting")]
        public string MoudelInfo = "模板信息";
        private bool ShowMoudleInfo => LycianthesConfig.GetInstance().UseModuleInfo;

        [LabelText("公司名")]
        [LabelWidth(LABEL_WIDTH)]
        [PropertyOrder(10)]
        [ShowInInspector, DisplayAsString]
        public string CompanyName
        {
            get { return LycianthesConfig.GetInstance().CompanyName; }
            set { }
        }

        [LabelText("用户名")]
        [LabelWidth(LABEL_WIDTH)]
        [ShowInInspector, DisplayAsString]
        [PropertyOrder(10)]
        public string AuthorName
        {
            get { return LycianthesConfig.GetInstance().AuthorName; }
            set { }
        }

        private void OnChangeUserSetting()
        {
            PlayerPrefs.SetString(KEY_USER_NAMESPACE, NameSpace);

            if (ShowMoudleInfo)
                PlayerPrefs.SetString(KEY_USER_MOUDLE_INFO, MoudelInfo);
        }

        [HideLabel, Title("脚本类型"), OnValueChanged("OnScriptTypeChanged")]
        [ValueDropdown("GetScriptKey")]
        [PropertyOrder(20)]
        public string scriptType = "CSharpCommon";
        private List<string> GetScriptKey() { return creator.GetScriptTemplateKey(); }

        [Title("保存位置"), Space(15)]
        [PropertyOrder(20)]
        [LabelWidth(LABEL_WIDTH), LabelText("批量创建")]
        public bool IsList = false;

        [LabelText("脚本名"), OnValueChanged("OnScriptNameChagned")]
        [PropertyOrder(20)]
        [LabelWidth(LABEL_WIDTH)]
        [HideIf("IsList")]
        public string ScriptName = "";


        [LabelText("脚本列表"), LabelWidth(LABEL_WIDTH), ShowIf("IsList")]
        [PropertyOrder(20)]
        public List<string> ListScriptName = new List<string>();

        [TabGroup("在当前文件夹创建")]
        [LabelWidth(LABEL_WIDTH)]
        [LabelText("创建位置"), FolderPath]
        [PropertyOrder(20)]
        public string TargetPath = "";

        [TabGroup("在默认文件夹创建")]
        [LabelWidth(LABEL_WIDTH)]
        [LabelText("默认位置"), FolderPath]
        [PropertyOrder(20)]
        public string DefaultPath = "";

        /// <summary>
        /// 当前缓存的脚本信息；
        /// </summary>
        private LycianthesScriptInfo tempInfo;

        /// <summary>
        /// 更新默认位置；
        /// </summary>
        public void OnScriptTypeChanged()
        {
            creator.dictTemplate.TryGetValue(scriptType, out tempInfo);
            if (tempInfo == null)
                tempInfo = new LycianthesScriptInfo();

            DefaultPath = tempInfo.DefaultFolder;

            if (!tempIsNameChanged)
            {
                ScriptName = tempInfo.DefaultName;
                tempIsNameChanged = false;
            }
        }

        private bool tempIsNameChanged = false;
        public void OnScriptNameChagned()
        { tempIsNameChanged = true; }

        [Button("开始创建"), HideIf("IsList")]
        [TabGroup("在当前文件夹创建")]
        [PropertyOrder(20)]
        public void CreateAtTargetPath()
        { CreateScript(ScriptName, TargetPath); }

        [Button("开始创建"), HideIf("IsList")]
        [TabGroup("在默认文件夹创建")]
        [PropertyOrder(20)]
        public void CreateAtDefaultPath()
        { CreateScript(ScriptName, DefaultPath); }

        public void CreateScript(string scriptName, string path)
        {
            if (string.IsNullOrEmpty(scriptName))
            {
                EditorUtility.DisplayDialog("警告", $"创建脚本 {scriptType}时出错：\n没有输入正确的脚本名！", "确定");
                return;
            }

            creator.Version = Version;
            creator.MoudleInfo = MoudelInfo;
            creator.NameSpace = NameSpace;
            //删除掉前面的Asset
            creator.CreateNewScript(scriptType, path.Substring(6), scriptName);
        }

        /// <summary>
        /// 批量创建；
        /// </summary>
        [Button("开始创建"), ShowIf("IsList")]
        [TabGroup("在当前文件夹创建")]
        [PropertyOrder(20)]
        public void CreateListScriptAtTarget()
        {
            int length = ListScriptName.Count;
            for (int i = 0; i < length; i++)
            { CreateScript(ListScriptName[i], TargetPath); }
        }

        /// <summary>
        /// 批量创建；
        /// </summary>
        [Button("开始创建"), ShowIf("IsList")]
        [TabGroup("在默认文件夹创建")]
        [PropertyOrder(20)]
        public void CreateListAtDefault()
        {
            int length = ListScriptName.Count;
            for (int i = 0; i < length; i++)
            { CreateScript(ListScriptName[i], DefaultPath); }
        }

        /// <summary>
        /// 刷新编辑器；
        /// </summary>
        [PropertySpace(15), Button("刷新")]
        [PropertyOrder(30)]
        public void Refresh() { AssetDatabase.Refresh(); }

    }
}
